prov_skill <- function(recipe, data, graph, type, Indices) {
  if (!Indices) {
    #Define parameters
    i <- switch(type,
                "fcst" = "parentnodename_fcst",
                "hcst" = "parentnodename_hcst",
                "obs" = "parentnodename_obs")

    #Particular case of computing Skill metrics for several Indices
    parentnodename <- graph$parentnodename[[i]]
    reference.nodename <- graph$parentnodename$parentnodename_obs
    indicesnodenames <- NULL
    #Change the nodename of the Verification step in
    #case this is not the only verification step!!

  }else {
    if (type == "hcst") {
      i <- length(graph$parentnodename$indicesnodenames)
      parentnodename <- graph$parentnodename$indicesnodenames[[1]]
      reference.nodename <- graph$parentnodename$indicesnodenames[[2]]
      graph$parentnodename$indicesnodenames <-
        graph$parentnodename$indicesnodenames[-c(1, 2)]
      indicesnodenames <- graph$parentnodename$indicesnodenames
    } else if (type == "obs") {
      parentnodename <- graph$parentnodename$indicesnodenames[[2]]
      indicesnodenames <- NULL
    }
  }
  graph <- graph$graph
  measure <- recipe$Analysis$Workflow$Skill$metric

  if (type == "hcst" || type == "fcst") {
    #Verification node
    if (!Indices) {
    verification.nodename <- paste0("Verification_", type)
    }else {
      verification.nodename <- paste0("Verification_", type, "_", i)
    }

    graph <- add_vertices(graph,
                          nv = 1,
                          name = verification.nodename,
                          label = "Verification",
                          className = "veri:ForecastVerification")

    #Linking the verification
    graph <- add_edges(graph,
                       c(getNodeIndexbyName(graph, parentnodename),
                         getNodeIndexbyName(graph, verification.nodename)),
                       label = "veri:hadValidation")

    #Linking the verification with the observations
    graph <- add_edges(graph,
                       c(getNodeIndexbyName(graph, reference.nodename),
                         getNodeIndexbyName(graph, verification.nodename)),
                       label = "veri:withReference")

    #Add Verification metrics
    measure <- unlist(strsplit(measure, " "))
    archive <- read_yaml("conf/variable-dictionary.yml")
    for (names in measure) {
      metric <- tolower(names)
      if (metric %in% names(archive$metrics)) {
        metric.method <- archive$metrics[[metric]]$long_name
        metric.class <- paste0("veri:",
                               archive$metrics[[metric]]$representation)
        description <- archive$metrics[[metric]]$definition
      }else {
        stop("Skill method not registered or not properly defined")
      }
      if (!Indices) {
        metric.nodename <- paste0("Metrics_", type, "_", metric)
      }else {
        metric.nodename <- paste0("Metrics_", type, "_", metric, "_", i)
      }

      graph <- add_vertices(graph,
                            nv = 1,
                            name = metric.nodename,
                            label = metric.method,
                            className = metric.class,
                            attr = list("dc:description" = description))

      graph <- add_edges(graph,
                         c(getNodeIndexbyName(graph, metric.nodename),
                           getNodeIndexbyName(graph, verification.nodename)),
                         label = "veri:withForecastRepresentation")
    }
    return(list("graph" = graph, "parentnodename" = verification.nodename,
                "indicesnodenames" = indicesnodenames))
  }else {
    return(list("graph" = graph, "parentnodename" = parentnodename,
                "indicesnodenames" = indicesnodenames))
  }
}