library(jsonlite)

write_rocrate <- function(expid) {
    cat("Starting write_rocrate function...\n")
    
    roc_dir <- paste0("/esarchive/autosubmit/", expid, "/ro-crate-metadata.json")
    
    # Check if file exists
    if (!file.exists(roc_dir)) {
        stop("Error: JSON file does not exist at ", roc_dir)
    }
    
    cat("Reading JSON file from:", roc_dir, "\n")
    rocrate <- fromJSON(roc_dir, simplifyVector = FALSE)
    
    # Add the new entity definition if it doesn't exist
    new_entity_id <- "https://w3id.org/ro/wfrun/provenance/0.5"
    entity_exists <- any(sapply(rocrate$`@graph`, function(x) x$`@id` == new_entity_id))
    
    if (!entity_exists) {
        new_entity <- list(
            `@id` = new_entity_id,
            `@type` = "CreativeWork",
            name = "Process Run Crate", 
            version = "0.5"
        )
        rocrate$`@graph` <- c(rocrate$`@graph`, list(new_entity))
    } else {
    }
    
    # Function to add the new profile to conformsTo
    add_profile_to_conformsTo <- function(entity) {
        if (!is.null(entity$conformsTo)) {
            exists_already <- any(sapply(entity$conformsTo, function(x) x$`@id` == new_entity_id))
            if (!exists_already) {
                entity$conformsTo <- c(entity$conformsTo, list(list(`@id` = new_entity_id)))
            }
        }
        return(entity)
    }
    
    # Update relevant entities
    for (i in seq_along(rocrate$`@graph`)) {
        if (rocrate$`@graph`[[i]]$`@id` %in% c("./", "ro-crate-metadata.json")) {
            rocrate$`@graph`[[i]] <- add_profile_to_conformsTo(rocrate$`@graph`[[i]])
        }
    }
    
    # Find and modify the experiment_data.yml entity
    for (i in seq_along(rocrate$`@graph`)) {
        entity <- rocrate$`@graph`[[i]]
        if (!is.null(entity$`@id`) && entity$`@id` == "conf/metadata/experiment_data.yml") {
            cat("Modifying experiment_data.yml entity...\n")
            
            if (!"HowTo" %in% entity$`@type`) {
                rocrate$`@graph`[[i]]$`@type` <- c(entity$`@type`, "HowTo")
            }
            
            how_to_step_ids <- list()
            for (step_entity in rocrate$`@graph`) {
                if (!is.null(step_entity$`@type`) && "HowToStep" %in% step_entity$`@type`) {
                    how_to_step_ids <- c(how_to_step_ids, list(list(`@id` = step_entity$`@id`)))
                }
            }
            
            if (length(how_to_step_ids) > 0) {
                rocrate$`@graph`[[i]]$step <- how_to_step_ids
            }
            break
        }
    }
    
    output_file <- paste0("/esarchive/autosubmit/", expid, "/ro-crate-metadata.json")
    
    cat("Writing modified JSON to:", output_file, "\n")
    write_json(rocrate, output_file, auto_unbox = TRUE, pretty = TRUE)
}

# Command-line execution
args <- commandArgs(trailingOnly = TRUE)
if (length(args) == 1) {
    write_rocrate(args[1])
} else {
    stop("Usage: Rscript modify_rocrate.R <expid>")
}
