write_rocrate <- function(recipes, out_dir, expid, repo_dir) {
  library(yaml)
  library(jsonlite)

  rocrate <- read_yaml(paste0(out_dir, "/outputs/provenance/rocrate.yml"))

  #Add modified entities that will need to be updated when merged to the full ro-crate-metadata.json
  new_profile_entity <- list(
    "@id" = "https://w3id.org/ro/wfrun/provenance/0.5",
    "@type" = "CreativeWork",
    name = "Process Run Crate", 
    version = "0.5"
  )

  rocrate$ROCRATE$PATCH$'@graph' <- c(rocrate$ROCRATE$PATCH$'@graph', list(new_profile_entity))

  # Add or find Organize Action entity
  org_action_idx <- which(sapply(rocrate$ROCRATE$PATCH$'@graph', function(x) x$'@id' == "#organize-action-sunset"))
  if (length(org_action_idx) == 0) {
    json_organize_action <- list(
      "@id" = "#organize-action-sunset",
      "@type" = "OrganizeAction",
      "agent" = list("@id" = rocrate$ROCRATE$PATCH$'@graph'[[1]]$creator$'@id'),
      "instrument" = list("@id" = "conf/metadata/experiment_data.yml"),
      "name" = "Run of SUNSET verification jobs",
      "object" = list(),
      "result" = "#create-action"
    )
    rocrate$ROCRATE$PATCH$'@graph' <- c(rocrate$ROCRATE$PATCH$'@graph', list(json_organize_action))
    org_action_idx <- length(rocrate$ROCRATE$PATCH$'@graph')
  }

  for (i in seq_along(recipes)) {
    recipe_name <- recipes[i]
    recipe <- paste0("atomic_recipe_", recipe_name, ".yml")
    recipe_dir <- paste0("/tmp/", basename(out_dir), "/logs/recipes/", recipe)
    json_name <- paste0("main_provenance_", recipe_name, ".json")
    json_dir <- paste0("/tmp/", basename(out_dir), "/outputs/provenance/", json_name)
    
    # Add control action reference to organize action
    control_id <- paste0("#control-action-sunset-", i)
    rocrate$ROCRATE$PATCH$'@graph'[[org_action_idx]]$object <- c(
      rocrate$ROCRATE$PATCH$'@graph'[[org_action_idx]]$object,
      list(list("@id" = control_id))
    )
    
    # Add new entities
    new_entities <- list(
      list(
        "@id" = control_id,
        "@type" = "ControlAction",
        "instrument" = list("@id" = paste0("#how-to-step-sunset-", i)),
        "name" = "Orchestrate SUNSET job run",
        "object" = list("@id" = paste0("#sunset-create-action-", i))
      ),
      list(
        "@id" = paste0("#how-to-step-sunset-", i),
        "@type" = "HowToStep",
        "workExample" = list("@id" = paste0("file:///", repo_dir))
      ),
      list(
        "@id" = paste0("#metaclip-create-action-", i),
        "@type" = "CreateAction",
        "instrument" = list("@id" = paste0("file://", repo_dir)),
        "name" = "SUNSET job run",
        "description" = "SUNSET job execution + Provenance based on METACLIP",
        "startTime" = format(Sys.time(), "%Y-%m-%d %H:%M"),
        "object" = list(list("@id" = recipe_dir)),
        "result" = list(list("@id" = json_dir))
      )
    )
    
    rocrate$ROCRATE$PATCH$'@graph' <- c(rocrate$ROCRATE$PATCH$'@graph', new_entities)
    rocrate$ROCRATE$OUTPUTS <- c(rocrate$ROCRATE$OUTPUTS, 
                                paste0("file://", json_dir),
                                paste0("file://", recipe_dir))
  }
  
  rocrate$ROCRATE$PATCH <- toJSON(rocrate$ROCRATE$PATCH, pretty = TRUE, auto_unbox = TRUE)

  # Remove rocrate.yml from SUNSET outputs folder
  rocrate_path <- paste0(out_dir, "/outputs/provenance/rocrate.yml")
  if (file.exists(rocrate_path)) {
    file.remove(rocrate_path)
}
  write_yaml(rocrate, paste0("/esarchive/autosubmit/", expid, "/conf/rocrate.yml"))
}